
// demo: simply echoes every packet with each byte incremented by 1 (but correct crc)
//
// version 20220106, Pieter-Tjerk de Boer, pa3fwm@pa3fwm.nl
//
// freely modifyable and distributable as this is intended as a demo and starting point / template


module top (
    input wire clk72MHz
    );


    // --------- PLL to make the 125 MHz clock -------------

    wire clk125MHz;
    EHXPLLL mypll (
       .CLKI(clk72MHz),
       .CLKFB(clk125MHz),
       .CLKOP(clk125MHz),
       .ENCLKOP(1),
       .RST(0),
    );
    defparam mypll.CLKI_DIV = "72";
    defparam mypll.CLKFB_DIV = "125";
    defparam mypll.CLKOP_DIV = "2";
    defparam mypll.CLKOP_ENABLE = "ENABLED";


    // --------- the SerDes with gige logic -------------

    wire [7:0] rxbyte;
    wire rxact;
    wire rxfin;
    wire rxclk;
    wire txclk;
    wire [7:0] txbyte;
    wire txact;
    reg txen=0;
    serdes_gbe eth0 (
       clk125MHz,
       rxbyte, rxclk, rxact, rxfin,
       txbyte, txclk, txen, txact,
    );


    // --------- handling received packets -------------

    reg [7:0] rxbuf [2047:0];
    reg [10:0] rxwptr=0;
    reg rxflag=0;

    always @(posedge rxclk) begin
       if (rxact) begin
          // write received bytes into buffer
          rxbuf[rxwptr] <= rxbyte;
          rxwptr <= rxwptr+1;
       end else begin
          rxwptr<=0;
       end
       if (rxfin) begin
          // tell the rest of the system we've received a packet
          rxflag<=~rxflag;
       end
    end

    // --------- transmitting packets -------------

    reg rxflag1=0;
    reg rxflag2=0;
    reg prevrxflag=0;
    reg [10:0] txptr=0;
    reg [7:0] txbyte0;
    reg [7:0] memout;
    assign txbyte = memout+1;  // we transmit the bytes read from the memory, incremented by 1
    always @ (posedge txclk) begin
       rxflag1 <= rxflag;
       rxflag2 <= rxflag1;
       prevrxflag <= rxflag2;
       if (rxflag2!=prevrxflag && !txen && !txact) begin
          // we've received a frame, let's send the first 64 bytes back!
          txen<=1;
       end
       if (!txact) txptr<=0; 
       else txptr<=txptr+1;
       if (txptr==64) begin
          // end of packet, finish transmission
          txen<=0;
       end 
       memout <= rxbuf[txptr];
    end

endmodule

